# Generated by Django 3.0 on 2019-12-26 21:17

import carrot.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('kallable', models.CharField(max_length=512)),
                ('args', carrot.fields.ListField(blank=True, default='')),
                ('kwargs', carrot.fields.DictField(blank=True, default='')),
                ('queue', models.CharField(blank=True, default='', max_length=255)),
                ('status', models.SmallIntegerField(choices=[(1, 'Pending'), (2, 'Running'), (3, 'Completed'), (4, 'Failed')], db_index=True, default=1)),
                ('message', models.CharField(default='', max_length=2048)),
                ('exit_code', models.SmallIntegerField(choices=[(-1, 'Notset'), (0, 'Success'), (1, 'Unknown Error')], default=-1)),
                ('created_by', models.CharField(blank=True, default='', max_length=512)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('started_on', models.DateTimeField(blank=True, null=True)),
                ('completed_on', models.DateTimeField(blank=True, null=True)),
            ],
        ),
    ]
