# Generated by Django 3.0 on 2020-03-24 23:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('carrot', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='exit_code',
            field=models.SmallIntegerField(blank=True, choices=[(0, 'Success'), (1, 'Unknown Error')], null=True),
        ),
        migrations.AlterField(
            model_name='task',
            name='queue',
            field=models.CharField(blank=True, default='default', max_length=255),
        ),
        migrations.AlterField(
            model_name='task',
            name='status',
            field=models.CharField(choices=[('pending', 'Pending'), ('running', 'Running'), ('completed', 'Completed'), ('failed', 'Failed')], default='pending', max_length=255),
        ),
    ]
